/*
 * Copyright (C) 2023 Hasl Developers
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <https://www.gnu.org/licenses/>.
 */

#ifndef HASL_MECHANISM_ANONYMOUS_H
#define HASL_MECHANISM_ANONYMOUS_H

#include <glib.h>
#include <glib-object.h>

#include <hasl/haslcontext.h>
#include <hasl/haslmechanism.h>
#include <hasl/haslversion.h>

G_BEGIN_DECLS

#define HASL_MECHANISM_ANONYMOUS_DOMAIN (g_quark_from_static_string("hasl-mechanism-anonymous"))

/**
 * HaslMechanismAnonymous:
 *
 * This mechanism implements the ANONYMOUS method described in
 * [RFC 4505](https://www.rfc-editor.org/rfc/rfc4505). It uses the value of
 * [property@Context:username] as the token.
 *
 * This mechanism is not secure, so you must set
 * [property@Context:allow-clear-text] to %TRUE if [property@Context:tls] is
 * %FALSE.
 *
 * Since: 0.2
 */

#define HASL_TYPE_MECHANISM_ANONYMOUS (hasl_mechanism_anonymous_get_type())

HASL_AVAILABLE_IN_0_2
G_DECLARE_FINAL_TYPE(HaslMechanismAnonymous, hasl_mechanism_anonymous, HASL,
                     MECHANISM_ANONYMOUS, HaslMechanism)

G_END_DECLS

#endif /* HASL_MECHANISM_ANONYMOUS_H */
